@@ -87,6 +87,17 @@ class Events(UserDict):
             out = cls(out, categories=cats)
         return out
 
+    def update(self, new_dict):
+        """Add all items in new_dict to self, overwrite existing items.
+        Same as python's dict.update but also keeps track of categories.
+
+        Args:
+            new_dict ([type]): [description]
+        """
+        super().update(new_dict)
+        if hasattr(self, 'categories') and hasattr(new_dict, 'categories'):
+            self.categories.update(new_dict.categories)
+
     def _init_df(self):
         return pd.DataFrame(columns=['name', 'start_seconds', 'stop_seconds'])
 
@@ -127,11 +138,30 @@ class Events(UserDict):
         df['stop_seconds'] = pd.to_numeric(df['stop_seconds'], errors='coerce')
         return df
 
-    def to_dataset(self):
-        df = self.to_df()
+    def to_lists(self, preserve_empty: bool = True):
+        """[summary]
+
+        Args:
+            preserve_empty (bool, optional):
+                In keeping with the convention that events have identical start and stop times and segments do not,
+                empty events are coded with np.nan as both start and stop and
+                empty segments are coded as np.nan as start and 0 as stop.
+                `from_df()` will obey this convention - if both start and stop are np.nan,
+                the name will be a segment,
+                if only the start is np.nan (the stop does not matter), the name will be an event
+                Defaults to True.
+
+        Returns:
+            Tuple[List[str], List[float], List[float]: with names, start_seconds, stop_seconds.
+        """
+        df = self.to_df(preserve_empty=preserve_empty)
         names = df.name.values
         start_seconds = df.start_seconds.values.astype(np.float)
         stop_seconds = df.stop_seconds.values.astype(np.float)
+        return names, start_seconds, stop_seconds
+
+    def to_dataset(self):
+        names, start_seconds, stop_seconds = self.to_lists()
 
         da_names = xr.DataArray(name='event_names', data=np.array(names, dtype='U128'), dims=['index',])
         da_times = xr.DataArray(name='event_times', data=np.array([start_seconds, stop_seconds]).T, dims=['index','event_time'], coords={'event_time': ['start_seconds', 'stop_seconds']})
@@ -207,6 +237,9 @@ class Events(UserDict):
 
     def delete_time(self, name, time, tol=0):
         nearest_start = self._find_nearest(self.start_seconds(name), time)
+        if nearest_start is None:
+            return []
+
         index = np.where(self.start_seconds(name) == nearest_start)[0][0]
 
         if self.categories[name] == 'segment':
@@ -221,7 +254,7 @@ class Events(UserDict):
             deleted_time = self[name][index, :]
             self[name] = np.delete(self[name], index, axis=0)
         else:
-            deleted_time = None
+            deleted_time = []
         return deleted_time
 
     def select_range(self, name: str, t0: Optional[float] = None, t1: Optional[float] = None, strict: bool = True):
@@ -289,10 +322,59 @@ class Events(UserDict):
         self[name] = np.delete(self[name], indices, axis=0)
         return len(indices)
 
+    def find_next(self, t: float, names: Optional[List[str]] = None):
+        """Find event starting after `t` of type in names.
+
+        Args:
+            t (float): Current time in seconds.
+            names (Optional[List[str]], optional):
+                List of event names to find next time in.
+                Defaults to None (search in all events of any name).
+
+        Returns:
+            float or None: time of nearest next event. None if there is no next.
+        """
+        if names is None:
+            names = self.names
+
+        nxt = []
+        for name in names:
+            cmp = self[name][:, 0] > t
+            if np.any(cmp):
+                nxt.append(self[name][np.argmax(cmp), 0])
+        if len(nxt):
+            return np.min(nxt)
+
+
+    def find_prev(self, t: float, names: Optional[List[str]] = None):
+        """Find event ending before `t` of type in names
+
+        Args:
+            t (float): Current time in seconds.
+            names (Optional[List[str]], optional):
+                List of event names to find prev time in.
+                Defaults to None (search in all events of any name).
+
+        Returns:
+            float or None: time of nearest previous event. None if there is no previous.
+        """
+        if names is None:
+            names = self.names
+
+        nxt = []
+        for name in names:
+            cmp = self[name][:, 1] < t
+            if np.any(cmp):
+                nxt.append(self[name][np.argmin(cmp)-1, 0])
+        if len(nxt):
+            return np.max(nxt) #* self.fs_song
+
     def _find_nearest(self, array, value):
-        array = np.asarray(array)
-        idx = (np.abs(array - value)).argmin()
-        return array[idx]
+        if not len(array):
+            return None
+        else:
+            idx = (np.abs(array - value)).argmin()
+            return array[idx]
 
     def _infer_categories(self):
         categories = dict()
