@@ -11,7 +11,7 @@ import skimage.draw
 import scipy.signal
 import time
 import logging
-from functools import partial
+from functools import partial, lru_cache
 from typing import Tuple
 
 from .. import xarray_behave as xb
@@ -38,7 +38,8 @@ class Model():
 class SegmentItem(pg.LinearRegionItem):
 
     def __init__(self, bounds: Tuple[float, float], event_index: int, xrange,
-                 time_bounds: Tuple[float, float]=None, movable=True, pen=None, brush=None, **kwargs):
+                 time_bounds: Tuple[float, float] = None, movable=True,
+                 pen=None, brush=None, **kwargs):
         """[summary]
 
         Args:
@@ -65,7 +66,8 @@ class SegmentItem(pg.LinearRegionItem):
 class EventItem(pg.InfiniteLine):
 
     def __init__(self, position: float, event_index: int, xrange,
-                 time_pos=None, movable=True, pen=None, **kwargs):
+                 time_pos=None, movable=True,
+                 pen=None, **kwargs):
         """[summary]
 
         Args:
@@ -85,7 +87,7 @@ class EventItem(pg.InfiniteLine):
             self.position = time_pos
         self.event_index = event_index
         self.xrange = xrange
-        # this corresponds to the undelrying only for TraveView, not for SpecView
+        # this corresponds to the undelrying only for TraceView, not for SpecView
         self._parent = self.getViewWidget
 
 
@@ -197,6 +199,7 @@ class TraceView(pg.PlotWidget):
         # this should be just a link/ref so changes in ds made by the controller will propagate
         # mabe make Model as thin wrapper around ds that also handles ion and use ref to Modle instance
         self.disableAutoRange()
+        self.enableAutoRange(False, False)
 
         # leave enought space so axes are aligned aligned
         y_axis = self.getAxis('left')
@@ -208,13 +211,15 @@ class TraceView(pg.PlotWidget):
 
         self.threshold_line = pg.InfiniteLine(movable=True, angle=0,
                                 pos=0,
-                                pen=pg.mkPen(color='r', width=1),
+                                pen=pg.mkPen(color='r', width=2, alpha=0.25),
                                 bounds=[0, None],
                                 label='Threshold',
                                 labelOpts={'position': 0.9})
 
+        self.annotation_items = []
+
     @property
-    def m(self):  # make read-only
+    def m(self):  # read-only property
         return self._m
 
     @property
@@ -236,44 +241,58 @@ class TraceView(pg.PlotWidget):
 
     def update_trace(self):
         self.clear()
+
+        # draw background channels
         if self.m.nb_channels is not None and self.m.show_all_channels and self.m.y_other is not None:
+            y_other = self.m.y_other[::int(self.m.step*2), :]
+            x_other = self.m.x[::int(self.m.step * 2)]
             for chan in range(self.m.nb_channels - 1):
-                        self.addItem(pg.PlotCurveItem(self.m.x[::self.m.step],
-                                                      self.m.y_other[::self.m.step, chan],
-                                                      pen=pg.mkPen(color=[127, 127, 127])))
-        # plot selected trace
-        self.addItem(pg.PlotCurveItem(self.m.x[::self.m.step],
-                                      np.array(self.m.y[::self.m.step])))
+                item = pg.PlotCurveItem(x_other, y_other[:, chan], pen=pg.mkPen(color=[127, 127, 127]))
+                self.addItem(item)
+
+        # plot active channel
+        x = self.m.x[::self.m.step]
+        item = pg.PlotCurveItem(x=x, y=np.array(self.m.y[::self.m.step], dtype=np.float), pen=pg.mkPen(color=[220, 220, 220], width=1))
+        self.addItem(item)
+
         self.autoRange(padding=0)
+
         # time of current frame in trace
-        self.addItem(pg.InfiniteLine(movable=False, angle=90,
-                                     pos=self.m.x[int(self.m.span / 2)],
-                                     pen=pg.mkPen(color='r', width=1)))
+        pos_line = pg.InfiniteLine(self.m.x[int(self.m.span / 2)], movable=False, angle=90,
+                                   pen=pg.mkPen(color='r', width=1))
+        self.addItem(pos_line)
+
+        # draw actice envelope and threshold
         if self.m.threshold_mode:
             self.addItem(self.threshold_line)
-            self.addItem(pg.PlotCurveItem(self.m.x[::self.m.step],
-                                          self.m.envelope[::self.m.step],
-                                          pen=pg.mkPen(color=[255, 127, 127])))
+            env_line = pg.PlotCurveItem(x=x, y=self.m.envelope[::self.m.step], pen=pg.mkPen(color=[196, 98, 98], width=1))
+            self.addItem(env_line)
 
-    def add_segment(self, onset, offset, region_typeindex, brush=None, movable=True):
+
+    def add_segment(self, onset, offset, region_typeindex, brush=None, pen=None, movable=True, text=None):
         region = SegmentItem((onset, offset), region_typeindex, self.xrange,
-                             brush=brush, movable=movable)
+                             brush=brush, pen=pen, movable=movable)
+        if text is not None:
+            pg.InfLineLabel(region.lines[1], text, position=0.95, rotateAxis=(1,0), anchor=(1, 1))
+
         self.addItem(region)
         if movable:
             region.sigRegionChangeFinished.connect(self.m.on_region_change_finished)
 
-    def add_event(self, xx, event_type, pen, movable=False):
+    def add_event(self, xx, event_type, pen, movable=False, text=None):
         if not len(xx):
             return
-        if not movable:
-            xx = np.broadcast_to(xx[np.newaxis, :], (2, len(xx)))
-            yy = np.zeros_like(xx) + self.yrange[:, np.newaxis]
-            utils.fast_plot(self, xx.T, yy.T, pen)
-        else:
-            for x in xx:
-                line = EventItem(x, event_type, self.xrange, movable=True, angle=90, pen=pen)
-                line.sigPositionChangeFinished.connect(self.m.on_position_change_finished)
-                self.addItem(line)
+        # if not movable:
+        #     xx = np.broadcast_to(xx[np.newaxis, :], (2, len(xx)))
+        #     yy = np.zeros_like(xx) + self.yrange[:, np.newaxis]
+        #     utils.fast_plot(self, xx.T, yy.T, pen)
+        # else:
+        for x in xx:
+            line = EventItem(x, event_type, self.xrange, movable=True, angle=90, pen=pen)
+            if text is not None:
+                pg.InfLineLabel(line, text, position=0.95, rotateAxis=(1,0), anchor=(1, 1))
+            line.sigPositionChangeFinished.connect(self.m.on_position_change_finished)
+            self.addItem(line)
 
     def time_to_pos(self, time):
         return np.interp(time, self.m.trange, self.xrange)
@@ -296,8 +315,9 @@ class SpecView(pg.ImageView):
         self.ui.roiBtn.hide()
         self.ui.menuBtn.hide()
         self.view.disableAutoRange()
+        self.view.setAspectLocked(False)
+        self.view.getViewBox().invertY(False)
         self.view.setMouseEnabled(x=False, y=False)
-
         # leave enought space so axes are aligned aligned
         y_axis = self.getView().getAxis('left')
         y_axis.setWidth(50)
@@ -308,6 +328,9 @@ class SpecView(pg.ImageView):
         self.callback = callback
         self.imageItem.mouseClickEvent = self._click
 
+        self.pos_line = pg.InfiniteLine(pos=0.5, movable=False, angle=90,
+                                        pen=pg.mkPen(color='r', width=1))
+        self.addItem(self.pos_line)
         if colormap is not None:
             self.imageItem.setLookupTable(colormap)  # apply the colormap
         self.old_items = []
@@ -319,7 +342,7 @@ class SpecView(pg.ImageView):
     @property
     def xrange(self):
         try:  # to prevent failure on init
-            return np.array(self.view.viewRange()[0])
+            return (0, self.S.shape[1]) #np.array(self.view.viewRange()[0])
         except AttributeError:
             return None
 
@@ -331,42 +354,39 @@ class SpecView(pg.ImageView):
             return None
 
     def clear_annotations(self):
-        [self.removeItem(item) for item in self.old_items]  # remove annotations
+        [self.removeItem(item) for item in self.old_items]  # remove annotations <- slowest part of update_spec!!!
 
     def update_spec(self, x, y):
         # hash x to avoid re-calculation? only useful when annotating
-        self.clear_annotations()
+        t0 = time.time()
+        # t1 = time.time()
+        # logging.debug(f'  update spec - clear annotations: {t1-t0}')
 
-        S, f, t = self._calc_spec(y)
-        self.setImage(S.T[:, ::-1])
-        self.view.setLimits(xMin=0, xMax=S.shape[1], yMin=0, yMax=S.shape[0])
+        # S, f, t = self._calc_spec(y)
+        S, f, t = self._calc_spec(tuple(y), self.m.spec_win)  # tuple-ify for caching
+        self.S = S
+        trange = (self.m.x[-1] - self.m.x[0])
+        t2 = time.time()
+        logging.debug(f'  update spec - calc spec: {t2-t0}')
 
-        f = f[::-1]
-        y_axis = self.getView().getAxis('left')
-        ticks = np.linspace(0, len(f)-1, 5, dtype=np.uintp)
+        self.setImage(S.T, autoRange=False, scale=[trange / len(t), (f[-1] - f[0]) / len(f)], pos=[self.m.x[0], f[0]])
+        self.view.setRange(xRange=self.m.x[[0, -1]], yRange=(f[0], f[-1]), padding=0)
 
-        tick_label = []
-        for ti in ticks:
-            tick = f[ti]
-            if tick > 10:
-                tick = int(tick)
-            else:
-                tick = round(tick, 1)
-            tick_label.append(str(tick))
-        y_axis.setTicks([[(pos, label) for pos, label in zip(ticks, tick_label)]])
+        self.pos_line.setValue(self.m.x[int(self.m.span / 2)])
 
-        x_axis = self.view.getAxis('bottom')
-        ticks = np.linspace(0, len(t)-1, 10, dtype=np.uintp)
-        t_offset = float(self.m.ds.sampletime[self.m.time0])
-        x_axis.setTicks([[(ii, str(int((t_offset + t[ii])*self.m.fs_song)/self.m.fs_song)) for ii in ticks]])
+        t3 = time.time()
+        logging.debug(f'  update spec - draw: {t3-t2}')
+        t4 = time.time()
+        logging.debug(f'update spec - TOTAL: {t4-t0}')
 
-    # @lru_cache(maxsize=2, typed=False)
-    def _calc_spec(self, y):
+    @lru_cache(maxsize=2, typed=False)
+    def _calc_spec(self, y, spec_win):
+        y = np.array(y)
         # signal.spectrogram will internally limit spec_win to len(y)
         # and will throw error since noverlap will then be too big
-        self.spec_win = min(len(y), self.m.spec_win)
-        f, t, psd = scipy.signal.spectrogram(y, self.m.fs_song, nperseg=self.spec_win,
-                                             noverlap=self.spec_win // 2, nfft=self.spec_win * 4, mode='magnitude')
+        spec_win = min(len(y), spec_win)
+        f, t, psd = scipy.signal.spectrogram(y, self.m.fs_song, nperseg=spec_win,
+                                             noverlap=spec_win // 2, nfft=spec_win * 4, mode='magnitude')
         self.spec_t = t
 
         if self.m.fmax is not None:
@@ -383,30 +403,35 @@ class SpecView(pg.ImageView):
         S = S / np.max(S) * 255  # normalize to 0...255
         return S, f[f_idx0:f_idx1], t
 
-    def add_segment(self, onset, offset, region_typeindex, brush=None, movable=True):
-        onset_spec, offset_spec = self.time_to_pos((onset, offset))
+    def add_segment(self, onset, offset, region_typeindex, brush=None, pen=None, movable=True, text=None):
+        onset_spec, offset_spec = onset, offset
         region = SegmentItem((onset_spec, offset_spec), region_typeindex, self.xrange,
-                             time_bounds=(onset, offset), brush=brush, movable=movable)
+                             time_bounds=(onset, offset), brush=brush, pen=pen, movable=movable)
+        if text is not None:
+            pg.InfLineLabel(region.lines[1], text, position=0.95, rotateAxis=(1,0), anchor=(1, 1))
+
         self.addItem(region)
         self.old_items.append(region)
         if movable:
             region.sigRegionChangeFinished.connect(self.m.on_region_change_finished)
 
-    def add_event(self, xx, event_type, pen, movable=False):
+    def add_event(self, xx, event_type, pen, movable=False, text=None):
         if not len(xx):
             return
         xx0 = xx.copy()
-        xx = self.time_to_pos(xx)
-        if not movable:
-            xx = np.broadcast_to(xx[np.newaxis, :], (2, len(xx)))
-            yy = np.zeros_like(xx) + self.yrange[:, np.newaxis]
-            self.old_items.append(utils.fast_plot(self, xx.T, yy.T, pen))
-        else:
-            for (x, x0) in zip(xx, xx0):
-                line = EventItem(x, event_type, self.xrange, time_pos=x0, movable=True, angle=90, pen=pen)
-                line.sigPositionChangeFinished.connect(self.m.on_position_change_finished)
-                self.addItem(line)
-                self.old_items.append(line)
+        # xx = self.time_to_pos(xx)
+        # if not movable:
+        #     xx = np.broadcast_to(xx[np.newaxis, :], (2, len(xx)))
+        #     yy = np.zeros_like(xx) + self.yrange[:, np.newaxis]
+        #     self.old_items.append(utils.fast_plot(self, xx.T, yy.T, pen))
+        # else:
+        for (x, x0) in zip(xx, xx0):
+            line = EventItem(x, event_type, self.xrange, time_pos=x0, movable=True, angle=90, pen=pen)
+            if text is not None:
+                pg.InfLineLabel(line, text, position=0.95, rotateAxis=(1,0), anchor=(1, 1))
+            line.sigPositionChangeFinished.connect(self.m.on_position_change_finished)
+            self.addItem(line)
+            self.old_items.append(line)
 
     def time_to_pos(self, time):
         return np.interp(time, self.m.trange, self.xrange)
@@ -477,6 +502,7 @@ class MovieView(utils.FastImageWidget):
                     self.fly_poses.setData(pos=None)
                     self.fly_poses.setVisible(False)
 
+            if 'pose_positions_allo' in self.m.ds or 'body_positions' in self.m.ds:
                 if self.m.show_dot:
                     frame = self.annotate_dot(frame)
                 else:
@@ -491,25 +517,32 @@ class MovieView(utils.FastImageWidget):
 
             self.setImage(frame, auto_scale=True)
             self.viewBox.setRange(xRange=y_range, yRange=x_range)
-            if self.m.show_framenumber:
-                self.image_view_framenumber_text.setPlainText(f'frame {self.m.framenumber}')
-            else:
-                self.image_view_framenumber_text.setPlainText('')
+            # if self.m.show_framenumber:
+            #     self.image_view_framenumber_text.setPlainText(f'frame {self.m.framenumber}')
+            # else:
+            #     self.image_view_framenumber_text.setPlainText('')
 
     def annotate_dot(self, frame):
         # mark each fly with uniquely colored dots
-        pos = np.array(self.m.ds.pose_positions_allo.data[self.m.index_other, :, self.m.thorax_index])
+        if 'pose_positions_allo' in self.m.ds:
+            pos = np.array(self.m.ds.pose_positions_allo.data[self.m.index_other, :, self.m.thorax_index])
+            cols = self.m.bodypart_colors[[2, 6]]
+        elif 'body_positions' in self.m.ds:
+            pos = np.array(self.m.ds.body_positions.data[self.m.index_other, :, 1])
+            cols = self.m.bodypart_colors[[0]]
         self.fly_positions.setData(pos=pos[:,::-1],
                                    symbolBrush=self.brushes, pen=None, size=10,
                                    acceptDrags=not self.m.move_poses,
                                    focalFly=self.m.focal_fly)
 
         # mark *focal* and *other* fly with circle
-        for this_fly, color in zip((self.m.focal_fly, self.m.other_fly), self.m.bodypart_colors[[2, 6]]):
-            fly_pos = self.m.ds.pose_positions_allo.data[self.m.index_other,
-                                                       this_fly,
-                                                       self.m.thorax_index].astype(np.uintp)
-            fly_pos = np.array(fly_pos)  # in case this is a dask.array
+        for this_fly, color in zip((self.m.focal_fly, self.m.other_fly), cols):
+            if 'pose_positions_allo' in self.m.ds:
+                fly_pos = self.m.ds.pose_positions_allo.data[self.m.index_other, this_fly, self.m.thorax_index]
+            elif 'body_positions' in self.m.ds:
+                fly_pos = self.m.ds.body_positions.data[self.m.index_other, this_fly, 1]
+
+            fly_pos = np.array(fly_pos).astype(np.uintp)  # in case this is a dask.array
             # only plot circle if fly is within the frame (also prevents overflow errors
             # for tracking errors that lead to VERY large position values)
             if fly_pos[0] <= frame.shape[0] and fly_pos[1] <= frame.shape[1]:
@@ -527,8 +560,11 @@ class MovieView(utils.FastImageWidget):
         return frame
 
     def crop_frame(self, frame):
-        fly_pos = self.m.ds.pose_positions_allo.data[self.m.index_other, self.m.focal_fly, self.m.thorax_index].astype(np.uintp)
-        fly_pos = np.array(fly_pos)  # in case this is a dask.array
+        if 'pose_positions_allo' in self.m.ds:
+            fly_pos = self.m.ds.pose_positions_allo.data[self.m.index_other, self.m.focal_fly, self.m.thorax_index]
+        elif 'body_positions' in self.m.ds:
+            fly_pos = self.m.ds.body_positions.data[self.m.index_other, self.m.focal_fly, 1]
+        fly_pos = np.array(fly_pos).astype(np.uintp)  # in case this is a dask.array
         # makes sure crop does not exceed frame bounds
         fly_pos[0] = np.clip(fly_pos[0], self.m.box_size, self.m.vr.frame_width - 1 - self.m.box_size)
         fly_pos[1] = np.clip(fly_pos[1], self.m.box_size, self.m.vr.frame_height - 1 - self.m.box_size)
